<?php
/**
 * @version		$Id: filegroup.class.php,v 1.18 2012-07-26 18:38:26 juanca Exp $
 * @package		VPL. class to edit a group of files
 * @copyright	2012 Juan Carlos Rodríguez-del-Pino
 * @license		http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 * @author		Juan Carlos Rodríguez-del-Pino <jcrodriguez@dis.ulpgc.es>
 */

require_once dirname(__FILE__).'/locallib.php';
require_once dirname(__FILE__).'/views/sh_factory.class.php';

class file_group_process{
	/**
	 * Name of file list
	 *
	 * @var string
	 */
	protected $filelistname;

	/**
	 * Path to directory where files are saved
	 *
	 * @var string
	 */
	protected $dir;

	/**
	 * Maximum number of files
	 *
	 * @var int
	 */
	protected $maxnumfiles;

	/**
	 * Number of files not changeables 0..($numstaticfiles-1)
	 *
	 * @var int
	 */
	protected $numstaticfiles;

	/**
	 * Constructor
	 *
	 * @param string $filelistname
	 * @param string $dir
	 * @param int $maxnumfiles
	 * @param int $numstaticfiles
	 */
	function __construct($filelistname,$dir,$maxnumfiles=10000,$numstaticfiles=0){
		$this->filelistname = $filelistname;
		$this->dir = $dir;
		if(strlen($dir) == 0 || $dir[strlen($dir)-1] != '/'){
			$this->dir .= '/';
		}
		$this->maxnumfiles = $maxnumfiles;
		$this->numstaticfiles = $numstaticfiles;
	}

	/**
	 * Get max number of files.
	 *
	 * @return int
	 */
	function get_maxnumfiles(){
		return $this->maxnumfiles;
	}

	/**
	 * Get number of static files.
	 *
	 * @return int
	 */
	function get_numstaticfiles(){
		return $this->numstaticfiles;
	}

	/**
	 * Add a new file to the group/Modify the data file
	 *
	 * @param string $filename
	 * @param string $data
	 * @return bool (added==true)
	 */
	function addFile($filename,$data=null){
		if($filename==''){
			return false;
		}
		ignore_user_abort (true);
		$filelist = $this->getFileList();
		foreach($filelist as $f){
			if($filename == $f){
				if($data !== null){
					$fd = vpl_fopen($this->dir.$filename);
					fwrite($fd,$data);
					fclose($fd);
				}
				return true;
			}
		}
		if(count($filelist)>= $this->maxnumfiles){
			return false;
		}
		$filelist[] = $filename;
		$this->setFileList($filelist);
		if($data){
			$fd = vpl_fopen($this->dir.$filename);
			fwrite($fd,$data);
			fclose($fd);
		}
		return true;
	}

	/**
	 * Delete a file from groupfile
	 *
	 * @param int $num file position
	 * @return bool
	 */
	function deleteFile($num){
		if($num < $this->numstaticfiles){
			return false;
		}
		ignore_user_abort (true);
		$filelist = $this->getFileList();
		$l = count($filelist);
		$ret = false;
		$filelistmod = array();
		for($i = 0 ;$i <$l; $i++){
			if($num== $i){
				$fullname = $this->dir.$filelist[$num];
				$ret = true;
				if(file_exists($fullname)){
					unlink($fullname);
				}
			}
			else{
				$filelistmod[]=$filelist[$i];
			}
		}
		if($ret){
			$this->setFileList($filelistmod);
		}
		return $ret;
	}

	/**
	 * Rename a file
	 *
	 * @param int $num
	 * @param string $filename new filename
	 * @return bool (renamed==true)
	 */
	function renameFile($num,$filename){
		if($num<$this->numstaticfiles || $filename == ''){
			return false;
		}
		ignore_user_abort (true);
		$filelist = $this->getFileList();
		if(array_search($filename,$filelist) !== false){
			return false;
		}
		if($num >= 0 && $num<count($filelist)){
			if(file_exists($this->dir.$filelist[$num])){
				rename($this->dir.$filelist[$num],$this->dir.$filename);
			}
			$filelist[$num] =$filename;
			$this->setFileList($filelist);
			return true;
		}else{
			return false;
		}
	}

	/**
	 * Get list of files
	 *
	 * @return string[]
	 */
	function getFileList(){
		return vpl_read_list_from_file($this->filelistname);
	}

	/**
	 * Set the file list.
	 *
	 * @param string[] $filelist
	 */
	function setFileList($filelist){
		vpl_write_list_to_file($this->filelistname,$filelist);
	}

	/**
	 * Get the file comment by number
	 *
	 * @param int $num
	 * @return string
	 */
	function getFileComment($num){
		return get_string('file').' '.($num+1);
	}

	/**
	 * Get the file data by number or name
	 *
	 * @param int/string $mix
	 * @return string
	 */
	function getFileData($mix){
		if(is_int($mix)){
			$num=$mix;
			$filelist = $this->getFileList();
			if($num>=0 && $num<count($filelist)){
				$filename =$this->dir.$filelist[$num];
				if(file_exists($filename)){
					return file_get_contents($filename);
				}else{
					return '';
				}
			}
		}
		elseif(is_string($mix)){
			$filename=basename($mix);
			$filelist = $this->getFileList();
			if(array_search($filename,$filelist)!== false){
				$fullfilename =$this->dir.$filename;
				if(file_exists($fullfilename)){
					return file_get_contents($fullfilename);
				}else{
					return '';
				}
			}
		}
		debugging("File not found $mix",DEBUG_DEVELOPER);
		return '';
	}

	/**
	 * Return is there is some file with data
	 * @return boolean
	 */
	function is_populated(){
		$filelist = $this->getFileList();
		foreach($filelist as $filename){
			$fullname = $this->dir.$filename;
			if(file_exists($fullname)){
				$info = stat($fullname);
			 	if($info['size']>0){
			 		return true;
			 	}
			}
		}
		return false;
	}


	/**
	 * Print file group
	 **/
	function print_files($if_no_exist=true){
		global $OUTPUT;
		$filenames = $this->getFileList();
		foreach ($filenames as $name) {
			if(file_exists($this->dir.$name)){
				echo '<h3>'.s($name).'</h3>';
				$printer= vpl_sh_factory::get_sh($name);
				echo $OUTPUT->box_start();
				$data = $this->getFileData($name);
				$printer->print_file($name,$data);
				echo $OUTPUT->box_end();
			}elseif($if_no_exist){
				echo '<h3>'.s($name).'</h3>';
			}
		}
	}
	function send_file($path, $filename, $lifetime = 'default' , $filter=0, $pathisstring=false, $forcedownload=false, $mimetype='') {
		global $CFG, $COURSE, $SESSION;
	
		// MDL-11789, apply $CFG->filelifetime here
		if ($lifetime === 'default') {
			if (!empty($CFG->filelifetime)) {
				$lifetime = $CFG->filelifetime;
			} else {
				$lifetime = 86400;
			}
		}
	
		// Use given MIME type if specified, otherwise guess it using mimeinfo.
		// IE, Konqueror and Opera open html file directly in browser from web even when directed to save it to disk :-O
		// only Firefox saves all files locally before opening when content-disposition: attachment stated
		$isFF         = check_browser_version('Firefox', '1.5'); // only FF > 1.5 properly tested
		$mimetype     = ($forcedownload and !$isFF) ? 'application/x-forcedownload' :
		($mimetype ? $mimetype : mimeinfo('type', $filename));
	
		// If the file is a Flash file and that the user flash player is outdated return a flash upgrader MDL-20841
		if (!empty($CFG->excludeoldflashclients) && $mimetype == 'application/x-shockwave-flash'&& !empty($SESSION->flashversion)) {
			$userplayerversion = explode('.', $SESSION->flashversion);
			$requiredplayerversion = explode('.', $CFG->excludeoldflashclients);
			if (($userplayerversion[0] <  $requiredplayerversion[0]) ||
					($userplayerversion[0] == $requiredplayerversion[0] && $userplayerversion[1] < $requiredplayerversion[1]) ||
					($userplayerversion[0] == $requiredplayerversion[0] && $userplayerversion[1] == $requiredplayerversion[1]
							&& $userplayerversion[2] < $requiredplayerversion[2])) {
				$path = $CFG->dirroot."/lib/flashdetect/flashupgrade.swf";  // Alternate content asking user to upgrade Flash
				$filename = "flashupgrade.swf";
				$lifetime = 0;  // Do not cache
			}
		}
	
		$lastmodified = $pathisstring ? time() : filemtime($path);
		$filesize     = $pathisstring ? strlen($path) : filesize($path);
	
		/* - MDL-13949
		 //Adobe Acrobat Reader XSS prevention
		if ($mimetype=='application/pdf' or mimeinfo('type', $filename)=='application/pdf') {
		//please note that it prevents opening of pdfs in browser when http referer disabled
		//or file linked from another site; browser caching of pdfs is now disabled too
		if (!empty($_SERVER['HTTP_RANGE'])) {
		//already byteserving
		$lifetime = 1; // >0 needed for byteserving
		} else if (empty($_SERVER['HTTP_REFERER']) or strpos($_SERVER['HTTP_REFERER'], $CFG->wwwroot)!==0) {
		$mimetype = 'application/x-forcedownload';
		$forcedownload = true;
		$lifetime = 0;
		} else {
		$lifetime = 1; // >0 needed for byteserving
		}
		}
		*/
	
		//IE compatibiltiy HACK!
		if (ini_get('zlib.output_compression')) {
			ini_set('zlib.output_compression', 'Off');
		}
	
		//try to disable automatic sid rewrite in cookieless mode
		@ini_set("session.use_trans_sid", "false");
	
		//do not put '@' before the next header to detect incorrect moodle configurations,
		//error should be better than "weird" empty lines for admins/users
		//TODO: should we remove all those @ before the header()? Are all of the values supported on all servers?
		header('Last-Modified: '. gmdate('D, d M Y H:i:s', $lastmodified) .' GMT');
	
		// if user is using IE, urlencode the filename so that multibyte file name will show up correctly on popup
		if (check_browser_version('MSIE')) {
			$filename = rawurlencode($filename);
		}
	
		if ($forcedownload) {
			@header('Content-Disposition: attachment; filename="'.$filename.'"');
		} else {
			@header('Content-Disposition: inline; filename="'.$filename.'"');
		}
	
		if ($lifetime > 0) {
			@header('Cache-Control: max-age='.$lifetime);
			@header('Expires: '. gmdate('D, d M Y H:i:s', time() + $lifetime) .' GMT');
			@header('Pragma: ');
	
			if (empty($CFG->disablebyteserving) && !$pathisstring && $mimetype != 'text/plain' && $mimetype != 'text/html') {
	
				@header('Accept-Ranges: bytes');
	
				if (!empty($_SERVER['HTTP_RANGE']) && strpos($_SERVER['HTTP_RANGE'],'bytes=') !== FALSE) {
					// byteserving stuff - for acrobat reader and download accelerators
					// see: http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.35
					// inspired by: http://www.coneural.org/florian/papers/04_byteserving.php
					$ranges = false;
					if (preg_match_all('/(\d*)-(\d*)/', $_SERVER['HTTP_RANGE'], $ranges, PREG_SET_ORDER)) {
						foreach ($ranges as $key=>$value) {
							if ($ranges[$key][1] == '') {
								//suffix case
								$ranges[$key][1] = $filesize - $ranges[$key][2];
								$ranges[$key][2] = $filesize - 1;
							} else if ($ranges[$key][2] == '' || $ranges[$key][2] > $filesize - 1) {
								//fix range length
								$ranges[$key][2] = $filesize - 1;
							}
							if ($ranges[$key][2] != '' && $ranges[$key][2] < $ranges[$key][1]) {
								//invalid byte-range ==> ignore header
								$ranges = false;
								break;
							}
							//prepare multipart header
							$ranges[$key][0] =  "\r\n--".BYTESERVING_BOUNDARY."\r\nContent-Type: $mimetype\r\n";
							$ranges[$key][0] .= "Content-Range: bytes {$ranges[$key][1]}-{$ranges[$key][2]}/$filesize\r\n\r\n";
						}
					} else {
						$ranges = false;
					}
					if ($ranges) {
						byteserving_send_file($path, $mimetype, $ranges);
					}
				}
			} else {
				/// Do not byteserve (disabled, strings, text and html files).
				@header('Accept-Ranges: none');
			}
		} else { // Do not cache files in proxies and browsers
			if (strpos($CFG->wwwroot, 'https://') === 0) { //https sites - watch out for IE! KB812935 and KB316431
				@header('Cache-Control: max-age=10');
				@header('Expires: '. gmdate('D, d M Y H:i:s', 0) .' GMT');
				@header('Pragma: ');
			} else { //normal http - prevent caching at all cost
				@header('Cache-Control: private, must-revalidate, pre-check=0, post-check=0, max-age=0');
				@header('Expires: '. gmdate('D, d M Y H:i:s', 0) .' GMT');
				@header('Pragma: no-cache');
			}
			@header('Accept-Ranges: none'); // Do not allow byteserving when caching disabled
		}
	
		if (empty($filter)) {
			if ($mimetype == 'text/html' && !empty($CFG->usesid) && empty($_COOKIE['MoodleSession'.$CFG->sessioncookie])) {
				//cookieless mode - rewrite links
				@header('Content-Type: text/html');
				$path = $pathisstring ? $path : implode('', file($path));
				$path = sid_ob_rewrite($path);
				$filesize = strlen($path);
				$pathisstring = true;
			} else if ($mimetype == 'text/plain') {
				@header('Content-Type: Text/plain; charset=utf-8'); //add encoding
			} else {
				@header('Content-Type: '.$mimetype);
			}
			@header('Content-Length: '.$filesize);
			while (@ob_end_flush()); //flush the buffers - save memory and disable sid rewrite
			if ($pathisstring) {
				echo $path;
			} else {
				readfile_chunked($path);
			}
		} else {     // Try to put the file through filters
			if ($mimetype == 'text/html') {
				$options = new object();
				$options->noclean = true;
				$options->nocache = true; // temporary workaround for MDL-5136
				$text = $pathisstring ? $path : implode('', file($path));
	
				$text = file_modify_html_header($text);
				$output = format_text($text, FORMAT_HTML, $options, $COURSE->id);
				if (!empty($CFG->usesid) && empty($_COOKIE['MoodleSession'.$CFG->sessioncookie])) {
					//cookieless mode - rewrite links
					$output = sid_ob_rewrite($output);
				}
	
				@header('Content-Length: '.strlen($output));
				@header('Content-Type: text/html');
				while (@ob_end_flush()); //flush the buffers - save memory and disable sid rewrite
				echo $output;
				// only filter text if filter all files is selected
			} else if (($mimetype == 'text/plain') and ($filter == 1)) {
				$options = new object();
				$options->newlines = false;
				$options->noclean = true;
				$text = htmlentities($pathisstring ? $path : implode('', file($path)));
				$output = '<pre>'. format_text($text, FORMAT_MOODLE, $options, $COURSE->id) .'</pre>';
				if (!empty($CFG->usesid) && empty($_COOKIE['MoodleSession'.$CFG->sessioncookie])) {
					//cookieless mode - rewrite links
					$output = sid_ob_rewrite($output);
				}
	
				@header('Content-Length: '.strlen($output));
				@header('Content-Type: text/html; charset=utf-8'); //add encoding
				while (@ob_end_flush()); //flush the buffers - save memory and disable sid rewrite
				echo $output;
			} else {    // Just send it out raw
				@header('Content-Length: '.$filesize);
				@header('Content-Type: '.$mimetype);
				while (@ob_end_flush()); //flush the buffers - save memory and disable sid rewrite
				if ($pathisstring) {
					echo $path;
				}else {
					readfile_chunked($path);
				}
			}
		}
		die; //no more chars to output!!!
	}
	
	/**
	 * Download files
	 * @parm $name name of zip file generated
	 **/
	function download_files($name){
		global $CFG;
		$zip = new ZipArchive();
		$zipfilename=tempnam($CFG->dataroot . '/temp/'  , 'vpl_zipdownload' );
		if($zip->open($zipfilename,ZIPARCHIVE::CREATE)){
			foreach ($this->getFileList() as $filename) {
				$zip->addFromString($filename, $this->getFileData($filename));
			}
			$zip->close();
			//Get zip data
			$data=file_get_contents($zipfilename);
			//remove zip file
			unlink($zipfilename);
			//Send zipdata
			@header('Content-Length: '.strlen($data));
			@header('Content-Disposition: attachment; filename="'.$name.'.zip"');
			@header('Cache-Control: private, must-revalidate, pre-check=0, post-check=0, max-age=0');
			@header('Expires: '. gmdate('D, d M Y H:i:s', 0) .' GMT');
			@header('Pragma: no-cache');
			@header('Accept-Ranges: none');
			echo $data;
		}
	}
}
?>
